from .functions import get_request

class Status():
    def __init__(self, arg_api_key : str):
        """
        클래스 생성 시 Neople Open API key를 입력받는다
            Args :
                arg_api_key(str) : Neople Open API key
        """
        self.__api_key = arg_api_key
        self.adventure_name = None
        self.guild_name = None
        self.adventure_level = None
        self.strength = None
        self.intellect = None
        self.health = None
        self.mentality = None
        self.fame = None

    def get_data(self, arg_server_id : str, arg_character_id : str):
        """
        캐릭터의 모험단명부터 명성등 정보를 반환한다
            Args:
                arg_server_id(str) :  서버 ID
                
                arg_character_id(str) : 캐릭터 ID
        """

        url = f'https://api.neople.co.kr/df/servers/{arg_server_id}/characters/{arg_character_id}/status?apikey={self.__api_key}'
        data = get_request(url)
        try:
            self.adventure_name = data['adventureName']
        except:
            pass
        try:
            self.guild_name = data['guildName']    
        except:
            pass
        try:
            self.adventure_level = data['buff'][0]['level']
        except:
            pass   
        try:             
            status_dict = dict()
            for item in data['status']:
                status_dict[item['name']] = item['value']                    
        except:
            pass        
        try:
            self.strength = status_dict["힘"]
        except:
            pass        
        try:
            self.intellect = status_dict["지능"]
        except:
            pass
        try:
            self.health = status_dict["체력"]
        except:
            pass                
        try:
            self.mentality = status_dict["정신력"]
        except:
            pass        
        try:
            self.fame = status_dict["모험가 명성"]
        except:
            pass        
        
        
        
        
        



import datetime
from .functions import get_request

class Timeline():
    
    def __init__(self, arg_api_key : str):
        """
        클래스 생성 시 Neople Open API key를 입력받는다
            Args :
                arg_api_key(str) : Neople Open API key
        """
        self.__api_key = arg_api_key
    
    def get_data(self, arg_server_id : str, arg_character_id : str, arg_end_date : str, arg_last_end_date : str = "2017-09-21 00:00", arg_last_end_data : dict or None = None, arg_limit : int = 100, arg_code : int or str = "", arg_print_log : bool = False):
        """
        서버ID와 캐릭터ID 원하는 수집시간(arg_end_date)을 입력받으면 타임라인데이터를 반환한다.
            Args :
                arg_server_id(str) : 서버ID ex) cain
                
                arg_character_id(str) : 캐릭터ID 
                
                arg_end_date(str) : 이 시간까지 수집을 한다 ex) 2023-03-03 15:57
                
                arg_last_end_date(str) : 이 시간부터 수집을 한다 ex) 2018-03-03 15:57
                
                arg_last_end_data(dict) : 지금까지 수집한 해당 캐릭터의 마지막 타임라인 데이터
                
                arg_limit(int) : 한번 request할 때 수집 할 타임라인 데이터의 개수
                
                arg_code(int) : 수집하고 싶은 타임라인 코드 ex)201, 202 참조) https://developers.neople.co.kr/contents/guide/pages/all 
                
                arg_print_log(boolean) : 데이터 수집의 과정의 print 여부   
        """
        self.timeline = []
        
        end_date = datetime.datetime.strptime(arg_end_date, '%Y-%m-%d %H:%M')
        start_date = end_date - datetime.timedelta(days=90)
        if start_date < datetime.datetime.strptime(arg_last_end_date, '%Y-%m-%d %H:%M'):
            start_date = datetime.datetime.strptime(arg_last_end_date, '%Y-%m-%d %H:%M')
        next = ""
        while start_date < end_date:
            stop = False
            url = f"""https://api.neople.co.kr/df/servers/{arg_server_id}/characters/{arg_character_id}/timeline?limit={arg_limit}&code={arg_code}&startDate={start_date.strftime('%Y-%m-%d %H:%M')}&endDate={end_date.strftime('%Y-%m-%d %H:%M')}&next={next}&apikey={self.__api_key}"""
            if arg_print_log:
                print(f"서버 = {arg_server_id}, 캐릭터 = {arg_character_id} 시작 = {start_date.strftime('%Y-%m-%d %H:%M')}, 끝 = {end_date.strftime('%Y-%m-%d %H:%M')}")
            data = get_request(url)
            next = data['timeline']['next']

            # 데이터가 있다면
            if data['timeline']['rows']:                 
                for log in data['timeline']['rows']:
                    if log == arg_last_end_data:
                        stop = True
                        break
                    else:
                        self.timeline.append(log)
                # 마지막으로 수집된 타임라인 데이터와 일치하는 항목이 있다면
                if stop:
                    break        

            # 타임라인데이터가 있고 마지막 로그가 캐릭터 생성이라면
            if self.timeline and self.timeline[-1]['code'] == 101:
                print("캐릭터 생성 로그를 확인했습니다")
                break

            # 해당기간에 next 데이터가 있으면
            if next:
                continue
            # 해당기간에 next 없으면
            else:
                end_date = start_date
                start_date = end_date - datetime.timedelta(days=90)
                if start_date < datetime.datetime.strptime(arg_last_end_date, '%Y-%m-%d %H:%M'):
                    start_date = datetime.datetime.strptime(arg_last_end_date, '%Y-%m-%d %H:%M')
                next = ""    
                continue                     




from .functions import get_request, one_slot
from .METADATA import AVATAR_LIST

class Avatar():
    def __init__(self):
        self.item_name = None
        self.item_rarity = None
        self.option_ability = None
        self.emblem_1 = None
        self.emblem_2 = None
        self.emblem_3 = None        
    def get_avatar_data(self, arg_avatar_dict):
        
        try:
            self.item_name = arg_avatar_dict["itemName"]
            self.item_rarity = arg_avatar_dict["itemRarity"]
        except:
            pass    
        
        try:
            self.option_ability = arg_avatar_dict["optionAbility"]
        except:
            pass
        
        try:    
            for index, emblem in enumerate(arg_avatar_dict['emblems']):
                exec(f"self.emblem_{index+1} = '{emblem['itemName']}'")
        except:
            pass        


class Avatars():
    def __init__(self, arg_api_key):
        """
        클래스 생성 시 Neople Open API key를 입력받는다
            Args :
                arg_api_key(str) : Neople Open API key
        """        
        self.__api_key = arg_api_key  
        for avatar in AVATAR_LIST:
            exec(f"self.{avatar} = Avatar()")  

    def get_data(self, arg_server_id, arg_character_id):    
        url = f'https://api.neople.co.kr/df/servers/{arg_server_id}/characters/{arg_character_id}/equip/avatar?apikey={self.__api_key}'
        data = get_request(url)
        data = data['avatar'] 
        for avatar in AVATAR_LIST:
            try:
                exec(f"self.{avatar}.get_avatar_data(one_slot(data, '{avatar}'.upper()))")
            except:
                pass    



# 초기 설정값
SETTINGS = {
    "request_time_out" : 5,
    "request_time_sleep" : 0.125
}

# 서버 NAME 이 key ID 가 value
SERVER_NAME_2_ID = {
    "안톤" : "anton",
    "바칼" : "bakal",
    "카인" : "cain",
    "카시야스" : "casillas",
    "디레지에" : "diregie",
    "힐더" : "hilder",
    "프레이" : "prey",
    "시로코" : "siroco"
    }

# 서버 ID 문자열 길이의 최대값
SERVERLENGTH = max(list(map(lambda x : len(x), list(SERVER_NAME_2_ID.values()))))

# 서버 ID 가 key NAME 이 value
SERVER_ID_2_NAME = {v : k for k , v in SERVER_NAME_2_ID.items()}

# 직업명
JOBCLASS = {
    "귀검사(남)" : ["웨펀마스터", "버서커", "소울브링어", "아수라", "검귀"],
    "격투가(남)" : ["넨마스터", "스트리트파이터", "그래플러", "스트라이커"],
    "거너(남)" : ["레인저", "메카닉", "런처", "스핏파이어", "어썰트"],
    "마법사(남)" : ["블러드 메이지", "엘레멘탈 바머", "빙결사", "디멘션워커", "스위프트 마스터"],
    "프리스트(남)" : ["크루세이더", "퇴마사", "인파이터", "어벤저"],
    "귀검사(여)" : ["소드마스터", "데몬슬레이어", "다크템플러", "베가본드", "블레이드"],
    "격투가(여)" : ["넨마스터", "스트리트파이터", "그래플러", "스트라이커"],
    "거너(여)" : ["레인저", "메카닉", "런처", "스핏파이어"],
    "마법사(여)" : ["엘레멘탈마스터", "마도학자", "소환사", "배틀메이지", "인챈트리스"],
    "프리스트(여)" : ["크루세이더", "이단심판관", "미스트리스", "무녀"],
    "도적" : ["로그", "쿠노이치", "섀도우댄서", "사령술사"],
    "나이트" : ["엘븐나이트", "카오스", "드래곤나이트", "팔라딘"],
    "마창사" : ["뱅가드", "듀얼리스트", "다크 랜서", "드래고니안 랜서"],
    "총검사" : ["요원", "트러블 슈터", "히트맨", "스페셜리스트"],
    "외전" : ["다크나이트", "크리에이터"],
    "아처" : ["뮤즈", "트래블러"]
}

jobclass_list = [item for sublist in list(JOBCLASS.values()) for item in sublist]

# 1차 전직명 문자열 길이의 최대값
JOB_GROW_NAME_LENGTH = max(list(map(lambda x : len(x), jobclass_list)))

# 직업명 문자열 길이의 최대값
JOB_NAME_LENGTH = max(list(map(lambda x : len(x), list(JOBCLASS.keys()))))

del jobclass_list

# 착용가능 장비
EQUIPMENT_LIST = ['weapon', 'title', 'jacket', 'shoulder', 'pants', 'shoes', 'waist', 'amulet', 'wrist', 'ring', 'support', 'magic_ston', 'earring']

# 착용가능 아바타
AVATAR_LIST = ['headgear', 'hair', 'face', 'jacket', 'pants', 'shoes', 'breast', 'waist', 'skin', 'aurora', 'weapon']

# 플래티넘 엠블렘 착용 가능 부위
PLATINUM_AVATAR_LIST = ['jacket', 'pants']


import time
import json
import requests
from .METADATA import JOBCLASS
from .METADATA import SETTINGS

__all__ = ['change_settings', 'get_request', 'jobname_equalize', 'get_job_info', 'system_maintenance']

def change_settings(arg_time_out : int, arg_time_sleep : float):
    """
    request에 필요한 설정 값을 바꾸는 함수

    초기 설정 값은 각각 5초, 0.125초
        Args :
            arg_time_out(int) : 해당 시간동안 응답이 없으면 에러 처리한다
            
            arg_time_sleep(float) : 한 번의 요청 후 쉬는 시간
    """
    global SETTINGS
    SETTINGS['request_time_out'] = arg_time_out
    SETTINGS['request_time_sleep'] = arg_time_sleep

def get_request(arg_url):
    """
    url 입력시 data 가져오는 함수
    arg_url : 원하는 url 주소
    """

    # request.get 실패 할 경우 에러 발생시킴
    try:
        data = requests.get(arg_url, timeout = SETTINGS['request_time_out'])
    except:
        raise Exception("해당 url주소는 사용할 수 없습니다")    
    
    # 성공한다면 api 규칙에 따라 지정한 시간을 멈춘다
    time.sleep(SETTINGS['request_time_sleep'])

    # 데이터 저장
    data = json.loads(data.text)

    # request.get은 성공 했으나 받아온 데이터가 Open API 에서 error로 규정된 경우 에러 발생시킴
    if list(data.keys())[0] == 'error':  
        raise Exception(f"apikey혹은 url에 문제가 있습니다. 코드를 확인해주세요. error : {data}")
    else:    
        return data
    

def _next(arg_dict, arg_list):
    """
    get_job_info 함수를 위해 쓰이는 함수
    """
    if 'next' in arg_dict.keys():
        arg_list.append(arg_dict["jobGrowName"])
        return _next(arg_dict['next'], arg_list)
    else :
        arg_list.append(arg_dict["jobGrowName"])
        return arg_dict["jobGrowName"]


def get_job_info(arg_api_key):
    """
    직업 정보를 받아오는 함수
    전직명(각성명)을 1차 전직명으로 통일시키는 jobname_equalize 함수에 매개변수로 사용되는 객체를 반환함
    arg_api_key : Neople Open API key
    """
    data = get_request(f"https://api.neople.co.kr/df/jobs?apikey={arg_api_key}")
    job_list = []
    jobGroW_total_list = []
    for job in data['rows']:
        job_list.append(job['jobName'])
        jobGroW_list = []
        for jobGroW in job['rows']:
            job_lo = []
            _next(jobGroW, job_lo)
            jobGroW_list.append(job_lo)
        jobGroW_total_list.append(jobGroW_list)        
    job_info = dict(zip(job_list, jobGroW_total_list))
    return job_info


def jobname_equalize(arg_job_name : str, arg_job_grow_name : str ,arg_job_info : dict):
    """
    직업명과 전직명을 받으면 해당 전직명으로 반환
        Args :
            arg_job_name(str) : 직업명 ex) 총검사
            arg_job_grow_name(str) : 전직명 ex) 빅보스
            arg_job_info(dict) : get_job_info 함수를 통해 받은 직업 정보
        Retruns :
            해당 직업의 1차 전직명 ex) 히트맨
    """
    if (arg_job_name in ["다크나이트", "크리에이터"]) or (arg_job_grow_name in list(JOBCLASS.keys())):
        output = arg_job_name
    else:    
        for job in arg_job_info[arg_job_name]:
            if arg_job_grow_name in job:
                output = job
                break
        output = output[0]
    return output

def system_maintenance(arg_api_key: str):
    """
    현재 Neople Open API 서버가 점검중인지 확인하는 함수\\
    서버가 점검중이면 TRUE를 반환한다
        Args :
            arg_api_key : Neople Open API key
        Retruns : 
            boolean
    """
    try:
        data = requests.get(f"https://api.neople.co.kr/df/servers?apikey={arg_api_key}", timeout = SETTINGS['request_time_out'])
        time.sleep(SETTINGS['request_time_sleep'])
        data = json.loads(data.text)
    except:
        return False    
    if (list(data.keys())[0] == 'error') and (data['error']['status'] == 503): 
        return True
    else:
        return False

def explain_enchant(arg_enchant_dict):
    """
    마법부여 정보를 정리해주는 함수
    """
    if arg_enchant_dict == {}:
        return None
    output = ""
    if "status" in arg_enchant_dict.keys():
        output = ", ".join([f"{s['name']} {s['value']}" for s in arg_enchant_dict['status']])
    if "reinforceSkill" in arg_enchant_dict.keys():
        output = ", ".join([f"{s['name']} {s['value']}" for r in arg_enchant_dict['reinforceSkill'] for s in r['skills']]) + ", " + output 
    if "explain" in arg_enchant_dict.keys():
        output = arg_enchant_dict['explain'] + ", " + output
    return output

def is_attr(arg_object):
    """
    하위 속성이 있는지 확인하는 함수
    """
    try:
        arg_object.__dict__.keys()
        return True
    except:
        return False

def get_attr(arg_object, te = ""):
    """
    객체의 하위 속성명을 list로 만들어주는 함수
    """
    st = []
    for sub in list(arg_object.__dict__.keys()):
        if is_attr(getattr(arg_object, sub)) == False:
            if sub[0] != "_":
                st.append(te + sub)
        else :     
            st.append(get_attr(getattr(arg_object, sub), te + sub + "."))    
    return st    

def get_values(arg_object):
    """
    객체의 하위 속성값을 list로 만들어주는 함수
    """
    st = []
    for sub in list(arg_object.__dict__.keys()):
        if is_attr(getattr(arg_object, sub)) == False:
            if sub[0] != "_":
                st.append(getattr(arg_object, sub))
        else:
            st.append(get_values(getattr(arg_object, sub)))    
    return st    

def flatten(arg_list):
    result = []
    for item in arg_list:
        if isinstance(item, list):
            result += flatten(item)
        else:
            result.append(item)
    return result

def attr_flatten(arg_object):
    """
    객체를 입력받으면 모든 하위속성의 이름을 문자열 list로 반환한다
    """
    arg_object = get_attr(arg_object)
    arg_object = flatten(arg_object)
    return arg_object

def value_flatten(arg_object):
    """
    객체를 입력받으면 모든 하위속성의 값을 list로 반환한다
    """
    arg_object = get_values(arg_object)
    arg_object = flatten(arg_object)
    return arg_object

def one_slot(arg_equipment_list : list, arg_slot_name : str):
    """
    해당 부위의 정보만 반환하는 함수
        Args:
            arg_equipment_list(list) : 캐릭터 장비 데이터(dict)로 이루어진 list
            
            arg_slot_name(str) : 해당 장비의 이름
        Returns:
            해당 장비의 dict            
    """
    if arg_equipment_list == []:
        return None
    for equipment in arg_equipment_list:
        if equipment['slotId'] == arg_slot_name :
            return equipment
    return None


from .functions import get_request, explain_enchant, one_slot
from .METADATA import EQUIPMENT_LIST

class OptionInfo():
    def __init__(self):
        self.explain = None
    
    def get_option_info_data(self, arg_option_info_dict):
        try:
            self.explain = arg_option_info_dict['explain'].replace("'", "") # 얼음 땡 옵션 예외처리를 위한 replace
        except:
            pass    

class GrowInfo():
    def __init__(self):
        self.level = None
        self.exp_rate = None        
        self.option_1 = OptionInfo()
        self.option_2 = OptionInfo()
        self.option_3 = OptionInfo()
        self.option_4 = OptionInfo()  
    def get_grow_info_data(self, arg_grow_info_dict):
        try: 
            self.level = arg_grow_info_dict['level']
            self.exp_rate = arg_grow_info_dict['expRate']
            for i in range(4):
                exec(f"self.option_{i+1}.get_option_info_data(arg_grow_info_dict['options'][{i}])")
        except:
            pass

class Equipment():
    def __init__(self):
        self.item_name = None
        self.item_available_level = None
        self.item_rarity = None
        self.reinforce = None
        self.item_grade_name = None
        self.enchant = None
        self.amplification_name = None
        self.refine = None
        self.upgrade_info = None
        self.grow_info = GrowInfo() 
    
    def get_equipment_data(self, arg_equipment_dict):
        try:
            self.item_name = arg_equipment_dict['itemName'].replace("'", "").lower() # 이름
            self.item_available_level = arg_equipment_dict['itemAvailableLevel'] # 레벨 제한
            self.item_rarity = arg_equipment_dict['itemRarity'] # 레어도
            self.reinforce = arg_equipment_dict['reinforce'] # 강화수치             
            self.amplification_name = arg_equipment_dict['amplificationName'] # 차원의 기운 여부 ex 차원의 힘, 차원의 지능, None
            self.refine = arg_equipment_dict['refine'] # 제련   
        except:
            pass    

        try:
            self.item_grade_name = arg_equipment_dict['itemGradeName'] # 등금(최상~최하)
        except:
            pass   

        try:
            self.enchant = explain_enchant(arg_equipment_dict['enchant']) # 마법부여
        except:
            pass         

        try:
            self.upgrade_info = arg_equipment_dict["upgradeInfo"]['itemName'] # 융합장비
        except:
            pass

        try:
            self.grow_info.get_grow_info_data(arg_equipment_dict["fixedOption"]) # 105제 성장 장비 정보(고정 에픽)
        except:
            pass

        try:
            self.grow_info.get_grow_info_data(arg_equipment_dict["customOption"]) # 105제 성장 장비 정보(커스텀 에픽)
        except:
            pass


class BakalInfo():
    def __init__(self):
        self.option_1 = None
        self.option_2 = None
        self.option_3 = None
    def get_bakal_info_data(self, arg_bakal_info_dict):        
        try:
            self.option_1 = arg_bakal_info_dict['options'][0]['explain']
        except:
            pass

        try:
            self.option_2 = arg_bakal_info_dict['options'][1]['explain']
        except:
            pass

        try:
            self.option_3 = arg_bakal_info_dict['options'][2]['explain']
        except:
            pass                

class Weapon(Equipment):
    def __init__(self):
        super().__init__()
        self.bakal_info = BakalInfo()

    def get_equipment_data(self, arg_equipment_dict):
        super().get_equipment_data(arg_equipment_dict)
        try:
            self.bakal_info.get_bakal_info_data(arg_equipment_dict["bakalInfo"]) # 바칼 무기 융합
        except:
            pass                


class Equipments():
    
    def __init__(self, arg_api_key : str):
        """
        클래스 생성 시 Neople Open API key를 입력받는다
            Args :
                arg_api_key(str) : Neople Open API key
        """
        self.__api_key = arg_api_key
        self.weapon = Weapon()
        for equipment in EQUIPMENT_LIST[1:]:
            exec(f"self.{equipment} = Equipment()")

    def get_data(self, arg_server_id : str, arg_character_id : str):
        url = f'https://api.neople.co.kr/df/servers/{arg_server_id}/characters/{arg_character_id}/equip/equipment?apikey={self.__api_key}'
        data = get_request(url)
        data = data['equipment']
        for equipment in EQUIPMENT_LIST:
            try:
                exec(f"self.{equipment}.get_equipment_data(one_slot(data, '{equipment}'.upper()))")
            except:
                pass                 
    



from .functions import get_request

class Flag():
    def __init__(self):
        self.item_rarity = None
        self.reinforce = None
        self.gem_1 = None
        self.gem_2 = None
        self.gem_3 = None
        self.gem_4 = None
    
    def get_flag_data(self, arg_flag_dict):
        try:
            self.item_rarity = arg_flag_dict['itemRarity']
            self.reinforce = arg_flag_dict['reinforce']
        except: 
            pass    
        try:
            arg_flag_dict = [gem['itemRarity'] for gem in arg_flag_dict['gems']]
        except:
            pass    
        for index, gem in enumerate(arg_flag_dict):
            try: 
                exec(f"self.gem_{index+1} = '{gem}'") 
            except:
                pass    

class Talismans():
    def __init__(self):
        self.item_name = None
        self.rune_1 = None
        self.rune_2 = None
        self.rune_3 = None
    def get_talismans_data(self, arg_talismans_dict):
        try: 
            self.item_name = arg_talismans_dict['talisman']['itemName']
        except:
            pass    
        try:
            arg_talismans_dict = [rune['itemName'] for rune in arg_talismans_dict['runes']]
        except:
            pass

        for index, rune in enumerate(arg_talismans_dict):
            try:
                exec(f"self.rune_{index+1} = '{rune}'")         
            except:
                pass    

class Others():
    def __init__(self, arg_api_key):
        """
        클래스 생성 시 Neople Open API key를 입력받는다
            Args :
                arg_api_key(str) : Neople Open API key
        """
        self.__api_key = arg_api_key
        self.creature = None
        self.flag = Flag()
        self.talisman_1 = Talismans()
        self.talisman_2 = Talismans()
        self.talisman_3 = Talismans()

    def get_data(self, arg_server_id, arg_character_id):
        
        
        # 크리쳐
        url = f"https://api.neople.co.kr/df/servers/{arg_server_id}/characters/{arg_character_id}/equip/creature?apikey={self.__api_key}"
        data = get_request(url)
        data = data["creature"]
        try:
            self.creature = data['itemName']
        except:
            pass    

        # 휘장
        url = f"https://api.neople.co.kr/df/servers/{arg_server_id}/characters/{arg_character_id}/equip/flag?apikey={self.__api_key}" 
        data = get_request(url)
        data = data["flag"]
        try:
            self.flag.get_flag_data(data)
        except:
            pass    

        # 탈리스만
        url = f"https://api.neople.co.kr/df/servers/{arg_server_id}/characters/{arg_character_id}/equip/talisman?apikey={self.__api_key}" 
        data = get_request(url)
        data = data["talismans"]
        try:
            for index, talismans_dict in enumerate(data):
                exec(f"self.talisman_{index+1}.get_talismans_data(talismans_dict)")
        except:
            pass    